'use strict';

var shim = require('use-sync-external-store/shim');

var __DEV__ = process.env.NODE_ENV !== 'production';
var isInAction = false;
var run = function (fn) {
    fn();
};
var resso = function (store) {
    if (__DEV__ && Object.prototype.toString.call(store) !== '[object Object]') {
        throw new Error('object required');
    }
    var state = {};
    var actions = {};
    Object.keys(store).forEach(function (key) {
        if (typeof store[key] === 'function') {
            var rawAction_1 = store[key];
            actions[key] = function () {
                var args = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    args[_i] = arguments[_i];
                }
                isInAction = true;
                var res = rawAction_1.apply(void 0, args);
                isInAction = false;
                return res;
            };
            return;
        }
        var listeners = new Set();
        state[key] = {
            subscribe: function (listener) {
                listeners.add(listener);
                return function () { return listeners.delete(listener); };
            },
            getSnapshot: function () { return store[key]; },
            setSnapshot: function (val) {
                if (val !== store[key]) {
                    store[key] = val;
                    run(function () { return listeners.forEach(function (listener) { return listener(); }); });
                }
            },
            useSnapshot: function () {
                return shim.useSyncExternalStore(state[key].subscribe, state[key].getSnapshot);
            },
        };
    });
    return new Proxy(store, {
        get: function (_, key) {
            if (key in actions) {
                return actions[key];
            }
            if (isInAction) {
                return store[key];
            }
            try {
                return state[key].useSnapshot();
            }
            catch (err) {
                return store[key];
            }
        },
        set: function (_, key, val) {
            if (key in store) {
                if (key in state) {
                    state[key].setSnapshot(val);
                }
                else if (__DEV__) {
                    throw new Error("'".concat(key, "' is a function, can not update"));
                }
            }
            else if (__DEV__) {
                throw new Error("'".concat(key, "' is not initialized in store"));
            }
            return true;
        },
    });
};
resso.config = function (_a) {
    var batch = _a.batch;
    run = batch;
};

module.exports = resso;
